@@ -9,7 +9,7 @@
     This file is part of mss.
 
     :copyright: Copyright 2019 Shivashis Padhi
-    :copyright: Copyright 2019-2020 by the mss team, see AUTHORS.
+    :copyright: Copyright 2019-2021 by the mss team, see AUTHORS.
     :license: APACHE-2.0, see LICENSE for details.
 
     Licensed under the Apache License, Version 2.0 (the "License");
@@ -231,6 +231,9 @@ def message_attachment():
     p_id = request.form.get("p_id", None)
     message_type = MessageType(int(request.form.get("message_type")))
     user = g.user
+    users = fm.fetch_users_without_permission(int(p_id), user.id)
+    if users is False:
+        return jsonify({"success": False, "message": "Could not send message. No file uploaded."})
     if file is not None:
         with fs.open_fs('/') as home_fs:
             file_dir = fs.path.join(APP.config['UPLOAD_FOLDER'], p_id)
@@ -278,9 +281,9 @@ def create_project():
     return str(fm.create_project(path, description, user, content=content))
 
 
-@APP.route('/get_project', methods=['GET'])
+@APP.route('/get_project_by_id', methods=['GET'])
 @verify_user
-def get_project():
+def get_project_by_id():
     p_id = request.values.get('p_id', None)
     user = g.user
     result = fm.get_file(int(p_id), user)
@@ -436,7 +439,7 @@ def modify_bulk_permissions():
     success = fm.modify_bulk_permission(p_id, user, u_ids, new_access_level)
     if success:
         for u_id in u_ids:
-            sockio.sm.emit_update_permission(u_id, p_id)
+            sockio.sm.emit_update_permission(u_id, p_id, access_level=new_access_level)
         sockio.sm.emit_project_permissions_updated(user.id, p_id)
         return jsonify({"success": True, "message": "User permissions successfully updated!"})
 
