@@ -28,11 +28,14 @@
     See the License for the specific language governing permissions and
     limitations under the License.
 """
+import os
+import sys
 import json
 import logging
 import types
 import fs
 import requests
+import re
 from fs import open_fs
 from werkzeug.urls import url_join
 
@@ -42,7 +45,9 @@ from mslib.msui import mscolab_project as mp
 from mslib.msui import mscolab_version_history as mvh
 from mslib.msui import sideview, tableview, topview
 from mslib.msui import socket_control as sc
-from mslib.msui.mss_qt import QtCore, QtGui, QtWidgets, get_open_filename
+
+from PyQt5 import QtCore, QtGui, QtWidgets
+from mslib.msui.mss_qt import get_open_filename, get_save_filename
 from mslib.msui.qt5 import ui_mscolab_help_dialog as msc_help_dialog
 from mslib.msui.qt5 import ui_add_project_dialog as add_project_ui
 from mslib.msui.qt5 import ui_add_user_dialog as add_user_ui
@@ -52,7 +57,6 @@ from mslib.msui.qt5 import ui_mscolab_merge_waypoints_dialog
 from mslib.utils import load_settings_qsettings, save_settings_qsettings, dropEvent, dragEnterEvent, show_popup
 from mslib.msui import constants
 from mslib.utils import config_loader
-from mslib.msui import MissionSupportSystemDefaultConfig as mss_default
 
 MSCOLAB_URL_LIST = QtGui.QStandardItemModel()
 
@@ -64,14 +68,17 @@ def add_mscolab_urls(combo_box, url_list):
 
 
 class MSSMscolabWindow(QtWidgets.QMainWindow, ui.Ui_MSSMscolabWindow):
-    """PyQt window implementing mscolab window
+    """
+    PyQt window implementing mscolab window
     """
     name = "Mscolab"
     identifier = None
     viewCloses = QtCore.pyqtSignal(name="viewCloses")
 
-    def __init__(self, parent=None, data_dir=mss_default.mss_dir, mscolab_server_url=mss_default.mscolab_server_url):
-        """Set up user interface
+    # ToDo refactor tests, mscolab_server_url not used
+    def __init__(self, parent=None, data_dir=None, mscolab_server_url=None):
+        """
+        Set up user interface
         """
         super(MSSMscolabWindow, self).__init__(parent)
         self.setupUi(self)
@@ -79,12 +86,11 @@ class MSSMscolabWindow(QtWidgets.QMainWindow, ui.Ui_MSSMscolabWindow):
         # if token is None, not authorized, else authorized
         self.token = None
         # User related signals
-        self.connectMscolab.clicked.connect(self.connect_handler)
+        self.toggleConnectionBtn.clicked.connect(self.connect_handler)
         self.addUser.clicked.connect(self.add_user_handler)
         self.loginButton.clicked.connect(self.authorize)
         self.logoutButton.clicked.connect(self.logout)
         self.deleteAccountButton.clicked.connect(self.delete_account)
-        self.disconnectMscolab.clicked.connect(self.disconnect_handler)
         self.helpBtn.clicked.connect(self.open_help_dialog)
         # Project related signals
         self.addProject.clicked.connect(self.add_project_handler)
@@ -132,22 +138,24 @@ class MSSMscolabWindow(QtWidgets.QMainWindow, ui.Ui_MSSMscolabWindow):
         # Mscolab help dialog
         self.help_dialog = None
         # set data dir, uri
-        self.data_dir = data_dir
+        if data_dir is None:
+            self.data_dir = config_loader(dataset="mss_dir")
+        else:
+            self.data_dir = data_dir
+        self.create_dir()
         self.mscolab_server_url = None
         self.disable_action_buttons()
         # disabling login, add user button. they are enabled when url is connected
         self.loginButton.setEnabled(False)
         self.addUser.setEnabled(False)
-        self.disconnectMscolab.setEnabled(False)
         self.url.setEditable(True)
         self.url.setModel(MSCOLAB_URL_LIST)
         # fill value of mscolab url from config
         default_MSCOLAB = config_loader(
-            dataset="default_MSCOLAB", default=mss_default.default_MSCOLAB)
+            dataset="default_MSCOLAB")
         add_mscolab_urls(self.url, default_MSCOLAB)
-
-        self.emailid.setText(config_loader(dataset="MSCOLAB_mailid", default=""))
-        self.password.setText(config_loader(dataset="MSCOLAB_password", default=""))
+        self.emailid.setEnabled(False)
+        self.password.setEnabled(False)
 
         # fill value of mscolab url if found in QSettings storage
         self.settings = \
@@ -156,13 +164,41 @@ class MSSMscolabWindow(QtWidgets.QMainWindow, ui.Ui_MSSMscolabWindow):
         if len(self.settings['recent_mscolab_urls']) > 0:
             add_mscolab_urls(self.url, self.settings['recent_mscolab_urls'])
 
+    def create_dir(self):
+        # ToDo this needs to be done earlier
+        if '://' in self.data_dir:
+            try:
+                _ = fs.open_fs(self.data_dir)
+            except fs.errors.CreateFailed:
+                logging.error(f'Make sure that the FS url "{self.data_dir}" exists')
+                show_popup(self, "Error", f'FS Url: "{self.data_dir}" does not exist!')
+                sys.exit()
+            except fs.opener.errors.UnsupportedProtocol:
+                logging.error(f'FS url "{self.data_dir}" not supported')
+                show_popup(self, "Error", f'FS Url: "{self.data_dir}" not supported!')
+                sys.exit()
+        else:
+            _dir = os.path.expanduser(self.data_dir)
+            if not os.path.exists(_dir):
+                os.makedirs(_dir)
+
     def disconnect_handler(self):
         self.logout()
+        self.status.setText("Status: disconnected")
+        self.emailid.setEnabled(False)
+        self.password.setEnabled(False)
         # enable and disable right buttons
-        self.disconnectMscolab.setEnabled(False)
         self.loginButton.setEnabled(False)
         self.addUser.setEnabled(False)
-        self.connectMscolab.setEnabled(True)
+        self.emailid.setEnabled(False)
+        self.password.setEnabled(False)
+        self.emailid.textChanged[str].disconnect(self.text_changed)
+        self.password.textChanged[str].disconnect(self.text_changed)
+        # toggle to connect button
+        self.toggleConnectionBtn.setText('Connect')
+        self.toggleConnectionBtn.clicked.disconnect(self.disconnect_handler)
+        self.toggleConnectionBtn.clicked.connect(self.connect_handler)
+        self.url.setEnabled(True)
         # set mscolab_server_url to None
         self.mscolab_server_url = None
 
@@ -184,18 +220,29 @@ class MSSMscolabWindow(QtWidgets.QMainWindow, ui.Ui_MSSMscolabWindow):
                 self.mscolab_server_url = url
                 self.status.setText("Status: connected")
                 # enable and disable right buttons
-                self.loginButton.setEnabled(True)
+                self.loginButton.setEnabled(False)
                 self.addUser.setEnabled(True)
-                self.disconnectMscolab.setEnabled(True)
-                self.connectMscolab.setEnabled(False)
+                self.emailid.setEnabled(True)
+                self.password.setEnabled(True)
+                # activate login button after email and password are entered
+                self.emailid.textChanged[str].connect(self.text_changed)
+                self.password.textChanged[str].connect(self.text_changed)
+                # toggle to disconnect button
+                self.toggleConnectionBtn.setText('Disconnect')
+                self.toggleConnectionBtn.clicked.disconnect(self.connect_handler)
+                self.toggleConnectionBtn.clicked.connect(self.disconnect_handler)
+                self.url.setEnabled(False)
                 if self.mscolab_server_url not in self.settings["server_settings"].keys():
                     self.settings["server_settings"].update({self.mscolab_server_url: {}})
-                try:
-                    recent_email = self.settings["server_settings"][self.mscolab_server_url]["recent_email"]
-                except KeyError:
-                    recent_email = ""
-                self.emailid.setText(recent_email)
                 save_settings_qsettings('mscolab', self.settings)
+                self.emailid.setEnabled(True)
+                self.password.setEnabled(True)
+                emailid = config_loader(dataset="MSCOLAB_mailid")
+                self.emailid.setText(emailid)
+                password = config_loader(dataset="MSCOLAB_password")
+                self.password.setText(password)
+                if len(emailid) > 0 and len(password) > 0:
+                    self.loginButton.setEnabled(True)
             else:
                 show_popup(self, "Error", "Some unexpected error occurred. Please try again.")
         except requests.exceptions.ConnectionError:
@@ -207,13 +254,19 @@ class MSSMscolabWindow(QtWidgets.QMainWindow, ui.Ui_MSSMscolabWindow):
         except requests.exceptions.InvalidURL:
             logging.debug("invalid url")
             show_popup(self, "Error", "Invalid URL")
+        except requests.exceptions.SSLError:
+            logging.debug("Certificate Verification Failed")
+            show_popup(self, "Error", "Certificate Verification Failed")
         except Exception as e:
             logging.debug("Error %s", str(e))
             show_popup(self, "Error", "Some unexpected error occurred. Please try again.")
 
+    def text_changed(self):
+        self.loginButton.setEnabled(self.emailid.text() != "" and self.password.text() != "")
+
     def handle_import(self):
-        file_path, _ = QtWidgets.QFileDialog.getOpenFileName(self, "Select a file", "", "Flight track (*.ftml)")
-        if file_path == "":
+        file_path = get_open_filename(self, "Select a file", "", "Flight track (*.ftml)")
+        if file_path is None:
             return
         dir_path, file_name = fs.path.split(file_path)
         with open_fs(dir_path) as file_dir:
@@ -234,9 +287,10 @@ class MSSMscolabWindow(QtWidgets.QMainWindow, ui.Ui_MSSMscolabWindow):
         show_popup(self, "Import Success", f"The file - {file_name}, was imported successfully!", 1)
 
     def handle_export(self):
-        file_path, _ = QtWidgets.QFileDialog.getSaveFileName(self, "Save Flight track", self.active_project_name,
-                                                             "Flight track (*.ftml)")
-        if file_path == "":
+        # Setting default filename path for filedialogue
+        default_filename = self.active_project_name + ".ftml"
+        file_path = get_save_filename(self, "Save Flight track", default_filename, "Flight track (*.ftml)")
+        if file_path is None:
             return
         xml_doc = self.waypoints_model.get_xml_doc()
         dir_path, file_name = fs.path.split(file_path)
@@ -257,6 +311,8 @@ class MSSMscolabWindow(QtWidgets.QMainWindow, ui.Ui_MSSMscolabWindow):
         self.versionHistoryBtn.setEnabled(False)
         self.deleteProjectBtn.setEnabled(False)
         self.helperTextLabel.setVisible(False)
+        self.emailid.setEnabled(False)
+        self.password.setEnabled(False)
 
     def disable_action_buttons(self):
         # disable some buttons to be activated after successful login or project activate
@@ -264,8 +320,7 @@ class MSSMscolabWindow(QtWidgets.QMainWindow, ui.Ui_MSSMscolabWindow):
         self.disable_project_buttons()
 
     def authenticate(self, data, r, url):
-        counter = 0
-        while r.status_code == 401 and counter < 5:
+        if r.status_code == 401:
             dlg = MSCOLAB_AuthenticationDialog(parent=self)
             dlg.setModal(True)
             if dlg.exec_() == QtWidgets.QDialog.Accepted:
@@ -277,7 +332,6 @@ class MSSMscolabWindow(QtWidgets.QMainWindow, ui.Ui_MSSMscolabWindow):
                 s.auth = (username, password)
                 s.headers.update({'x-test': 'true'})
                 r = s.post(url, data=data)
-                counter += 1
         return r
 
     def add_project_handler(self):
@@ -325,6 +379,11 @@ class MSSMscolabWindow(QtWidgets.QMainWindow, ui.Ui_MSSMscolabWindow):
             self.error_dialog = QtWidgets.QErrorMessage()
             self.error_dialog.showMessage('Description can\'t be empty')
             return
+        # regex checks if the whole path from beginning to end only contains alphanumerical characters or _ and -
+        elif not re.match("^[a-zA-Z0-9_-]*$", path):
+            self.error_dialog = QtWidgets.QErrorMessage()
+            self.error_dialog.showMessage('Path can\'t contain spaces or special characters')
+            return
 
         data = {
             "token": self.token,
@@ -333,7 +392,7 @@ class MSSMscolabWindow(QtWidgets.QMainWindow, ui.Ui_MSSMscolabWindow):
         }
         if self.add_proj_dialog.f_content is not None:
             data["content"] = self.add_proj_dialog.f_content
-        r = requests.post('{}/create_project'.format(self.mscolab_server_url), data=data)
+        r = requests.post(f'{self.mscolab_server_url}/create_project', data=data)
         if r.text == "True":
             self.error_dialog = QtWidgets.QErrorMessage()
             self.error_dialog.showMessage('Your project was created successfully')
@@ -352,7 +411,7 @@ class MSSMscolabWindow(QtWidgets.QMainWindow, ui.Ui_MSSMscolabWindow):
         self.user_diag.show()
 
     def add_user(self):
-        for key, value in config_loader(dataset="MSC_login", default={}).items():
+        for key, value in config_loader(dataset="MSC_login").items():
             if key not in constants.MSC_LOGIN_CACHE:
                 constants.MSC_LOGIN_CACHE[key] = value
         auth = constants.MSC_LOGIN_CACHE.get(self.mscolab_server_url, (None, None))
@@ -370,7 +429,7 @@ class MSSMscolabWindow(QtWidgets.QMainWindow, ui.Ui_MSSMscolabWindow):
             s = requests.Session()
             s.auth = (auth[0], auth[1])
             s.headers.update({'x-test': 'true'})
-            url = '{}/register'.format(self.mscolab_server_url)
+            url = f'{self.mscolab_server_url}/register'
             r = s.post(url, data=data)
             if r.status_code == 401:
                 r = self.authenticate(data, r, url)
@@ -381,7 +440,7 @@ class MSSMscolabWindow(QtWidgets.QMainWindow, ui.Ui_MSSMscolabWindow):
                 self.error_dialog.showMessage('You are registered, you can now log in.')
             else:
                 self.error_dialog = QtWidgets.QErrorMessage()
-                self.error_dialog.showMessage(r.json()["message"])
+                self.error_dialog.showMessage('Oh no, server authentication were incorrect.')
         else:
             self.error_dialog = QtWidgets.QErrorMessage()
             self.error_dialog.showMessage('Oh no, your passwords don\'t match')
@@ -495,8 +554,10 @@ class MSSMscolabWindow(QtWidgets.QMainWindow, ui.Ui_MSSMscolabWindow):
             if self.version_window is not None:
                 self.version_window.close()
             self.create_local_project_file()
-            self.local_ftml_file = fs.path.join(self.data_dir, 'local_mscolab_data',
-                                                self.user['username'], self.active_project_name, 'mscolab_project.ftml')
+            self.local_ftml_file = fs.path.combine(self.data_dir,
+                                                   fs.path.join('local_mscolab_data',
+                                                                self.user['username'], self.active_project_name,
+                                                                'mscolab_project.ftml'))
             self.helperTextLabel.setText(
                 self.tr("Working On: Local File. Your changes are only available to you."
                         "To save your changes with everyone, use the \"Save to Server\" button."))
@@ -519,7 +580,7 @@ class MSSMscolabWindow(QtWidgets.QMainWindow, ui.Ui_MSSMscolabWindow):
         self.reload_view_windows()
 
     def authorize(self):
-        for key, value in config_loader(dataset="MSC_login", default={}).items():
+        for key, value in config_loader(dataset="MSC_login").items():
             if key not in constants.MSC_LOGIN_CACHE:
                 constants.MSC_LOGIN_CACHE[key] = value
         auth = constants.MSC_LOGIN_CACHE.get(self.mscolab_server_url, (None, None))
@@ -534,13 +595,23 @@ class MSSMscolabWindow(QtWidgets.QMainWindow, ui.Ui_MSSMscolabWindow):
         s.auth = (auth[0], auth[1])
         s.headers.update({'x-test': 'true'})
         url = self.mscolab_server_url + '/token'
-        r = s.post(url, data=data)
+        try:
+            r = s.post(url, data=data)
+        except requests.exceptions.ConnectionError as ex:
+            logging.error("unexpected error: %s %s %s", type(ex), url, ex)
+            # popup that Failed to establish a connection
+            self.error_dialog = QtWidgets.QErrorMessage()
+            self.error_dialog.showMessage('Failed to establish a new connection'
+                                          f' to "{self.mscolab_server_url}". Try in a moment again.')
+            return
         if r.status_code == 401:
             r = self.authenticate(data, r, url)
-            if r.status_code == 200:
+            if r.status_code == 200 and not r.text == "False":
                 constants.MSC_LOGIN_CACHE[self.mscolab_server_url] = (auth[0], auth[1])
-                self.after_authorize(emailid, r)
-        elif r.text == "False":
+            else:
+                self.error_dialog = QtWidgets.QErrorMessage()
+                self.error_dialog.showMessage('Oh no, server authentication were incorrect.')
+        if r.text == "False" or r.text == "Unauthorized Access":
             # popup that has wrong credentials
             self.error_dialog = QtWidgets.QErrorMessage()
             self.error_dialog.showMessage('Oh no, your credentials were incorrect.')
@@ -553,7 +624,6 @@ class MSSMscolabWindow(QtWidgets.QMainWindow, ui.Ui_MSSMscolabWindow):
         self.token = _json["token"]
         self.user = _json["user"]
         self.label.setText(self.tr(f"Welcome, {self.user['username']}"))
-        self.password.setText("")
         self.loggedInWidget.show()
         self.loginWidget.hide()
         self.add_projects()
@@ -566,7 +636,6 @@ class MSSMscolabWindow(QtWidgets.QMainWindow, ui.Ui_MSSMscolabWindow):
         self.conn.signal_project_deleted.connect(self.handle_project_deleted)
         # activate add project button here
         self.addProject.setEnabled(True)
-        self.settings['server_settings'][self.mscolab_server_url].update({"recent_email": emailid})
         save_settings_qsettings('mscolab', self.settings)
 
     def add_projects(self):
@@ -582,7 +651,6 @@ class MSSMscolabWindow(QtWidgets.QMainWindow, ui.Ui_MSSMscolabWindow):
     def get_recent_pid(self):
         """
         get most recent project's p_id
-        # ToDo can be merged with get_recent_project
         """
         data = {
             "token": self.token
@@ -622,7 +690,7 @@ class MSSMscolabWindow(QtWidgets.QMainWindow, ui.Ui_MSSMscolabWindow):
         self.listProjects.itemActivated.connect(self.set_active_pid)
 
     def force_close_view_windows(self):
-        for window in self.active_windows:
+        for window in self.active_windows[:]:
             window.handle_force_close()
         self.active_windows = []
 
@@ -659,6 +727,8 @@ class MSSMscolabWindow(QtWidgets.QMainWindow, ui.Ui_MSSMscolabWindow):
 
         if self.access_level == "viewer" or self.access_level == "collaborator":
             if self.access_level == "viewer":
+                self.workLocallyCheckBox.setEnabled(False)
+                self.importBtn.setEnabled(False)
                 self.chatWindowBtn.setEnabled(False)
             else:
                 self.chatWindowBtn.setEnabled(True)
@@ -690,7 +760,7 @@ class MSSMscolabWindow(QtWidgets.QMainWindow, ui.Ui_MSSMscolabWindow):
             "token": self.token,
             "p_id": self.active_pid
         }
-        r = requests.get(self.mscolab_server_url + '/get_project', data=data)
+        r = requests.get(self.mscolab_server_url + '/get_project_by_id', data=data)
         xml_content = json.loads(r.text)["content"]
         return xml_content
 
@@ -794,6 +864,8 @@ class MSSMscolabWindow(QtWidgets.QMainWindow, ui.Ui_MSSMscolabWindow):
 
     def logout(self):
         self.clean_up_window()
+        self.emailid.setEnabled(True)
+        self.password.setEnabled(True)
 
     def delete_account(self):
         w = QtWidgets.QWidget()
@@ -851,6 +923,7 @@ class MSSMscolabWindow(QtWidgets.QMainWindow, ui.Ui_MSSMscolabWindow):
         server_xml = self.request_wps_from_server()
         server_waypoints_model = ft.WaypointsTableModel(xml_content=server_xml)
         self.merge_dialog = MscolabMergeWaypointsDialog(self.waypoints_model, server_waypoints_model, parent=self)
+        self.merge_dialog.saveBtn.setDisabled(True)
         if self.merge_dialog.exec_():
             xml_content = self.merge_dialog.get_values()
             if xml_content is not None:
@@ -884,6 +957,7 @@ class MSSMscolabWindow(QtWidgets.QMainWindow, ui.Ui_MSSMscolabWindow):
         server_xml = self.request_wps_from_server()
         server_waypoints_model = ft.WaypointsTableModel(xml_content=server_xml)
         self.merge_dialog = MscolabMergeWaypointsDialog(self.waypoints_model, server_waypoints_model, True, self)
+        self.merge_dialog.saveBtn.setDisabled(True)
         if self.merge_dialog.exec_():
             xml_content = self.merge_dialog.get_values()
             if xml_content is not None:
@@ -965,16 +1039,16 @@ class MSSMscolabWindow(QtWidgets.QMainWindow, ui.Ui_MSSMscolabWindow):
             self.close_external_windows()
             self.disable_project_buttons()
 
-            # Update project list
-            remove_item = None
-            for i in range(self.listProjects.count()):
-                item = self.listProjects.item(i)
-                if item.p_id == p_id:
-                    remove_item = item
-            if remove_item is not None:
-                logging.debug("remove_item: %s" % remove_item)
-                self.listProjects.takeItem(self.listProjects.row(remove_item))
-                return remove_item.text().split(' - ')[0]
+        # Update project list
+        remove_item = None
+        for i in range(self.listProjects.count()):
+            item = self.listProjects.item(i)
+            if item.p_id == p_id:
+                remove_item = item
+        if remove_item is not None:
+            logging.debug("remove_item: %s" % remove_item)
+            self.listProjects.takeItem(self.listProjects.row(remove_item))
+            return remove_item.text().split(' - ')[0]
 
     @QtCore.Slot(int, int)
     def handle_revoke_permission(self, p_id, u_id):
@@ -1092,7 +1166,10 @@ class MscolabMergeWaypointsDialog(QtWidgets.QDialog, ui_mscolab_merge_waypoints_
             row = deselected.indexes()[index].row()
             delete_waypoint = wp_dict[row]
             self.merge_waypoints_list.remove(delete_waypoint)
-
+        if len(self.merge_waypoints_list) > 1:
+            self.saveBtn.setDisabled(False)
+        else:
+            self.saveBtn.setDisabled(True)
         self.merge_waypoints_model = ft.WaypointsTableModel(waypoints=self.merge_waypoints_list)
         self.mergedWaypointsTable.setModel(self.merge_waypoints_model)
 
@@ -1120,7 +1197,8 @@ class MSCOLAB_AuthenticationDialog(QtWidgets.QDialog, ui_pw.Ui_WMSAuthentication
         self.setupUi(self)
 
     def getAuthInfo(self):
-        """Return the entered username and password.
+        """
+        Return the entered username and password.
         """
         return (self.leUsername.text(),
                 self.lePassword.text())
